from importlib import import_module

from django.conf import settings


def autodiscover():
    if not hasattr(settings, "EVENT_APPS"):
        return

    if not isinstance(settings.EVENT_APPS, list):
        raise Exception("EVENT_APPS should be a list")

    for app in settings.EVENT_APPS:
        # load processors
        try:
            import_module("%s.%s" % (app, "events.processors"))
        except Exception as e:
            print(e)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class EventsManagerConfig(AppConfig):
    name = "events_framework"
    verbose_name = _("Events Framework")
    default = True

    def ready(self) -> None:
        super().ready()
        self.module.autodiscover()



from .manager import manager


def register(event_model, event_type):
    def middle(func):
        manager.register(event_model, event_type, func)

        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)

        return wrapper

    return middle



from django import VERSION
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

if VERSION[0] == 2:
    from jsonfield import JSONField
else:
    from django.db.models.fields.json import JSONField


class EventModel(models.Model):
    date = models.DateTimeField(
        _("Date"),
        default=timezone.now,
        editable=False,
    )

    parameters = JSONField(
        _("Parameters"),
        blank=True,
        default=dict,
    )

    processed = models.BooleanField(
        _("Proccesed"),
        default=False,
    )

    error = models.BooleanField(
        _("Has error?"),
        default=False,
    )

    error_message = models.TextField(_("Error message"))

    class Meta:
        abstract = True
        ordering = ("date",)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(type={self.type!r})"




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="django-events-framework",
    version="0.0.9",
    author="Alexandru Gheorghita",
    author_email="gheorghitacristian@mac.com",
    description="An events framework for logging and processing Django models events.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    entry_points="",
    url="https://github.com/Reve/django-events-framework",
    packages=setuptools.find_packages(exclude=("test_app*",)),
    include_package_data=True,
    install_requires=["django>=2.2", "django-jsonfield"],
    python_requires=">=3.6",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Utilities",
    ],
)



import logging
from django.db import transaction

logger = logging.getLogger(__name__)


class EventsManager:
    def __init__(self):
        self._events_registry = {}

    def register(self, event_model, event_type, handler):
        if handler in self._events_registry:
            logger.info("event handler allready in registry")
            return

        self._events_registry[handler] = {
            "type": event_type,
            "event_model": event_model,
        }

    def process(self):
        for handler, props in self._events_registry.items():
            event_model = props["event_model"]
            event_type = props["type"]
            events_to_process = event_model.objects.filter(
                type=event_type,
                processed=False,
            )

            for event in events_to_process:
                with transaction.atomic():
                    for e in events_to_process.filter(pk=event.pk).select_for_update(
                        skip_locked=True
                    ):
                        try:
                            handler(e)
                            e.processed = True
                            e.save()
                        except Exception as e:
                            logger.error(str(e))


manager = EventsManager()
